@@ -22,7 +22,7 @@
  
 /*!
 	\author Steve Lhomme
-	\version \$Id: ACMStream.cpp,v 1.7 2002/06/12 18:36:27 markt Exp $
+	\version \$Id: ACMStream.cpp,v 1.9 2004/01/05 16:19:52 bouvigne Exp $
 */
 
 #if !defined(STRICT)
@@ -94,7 +94,7 @@ DebugFileName[0] = '\0';
 ACMStream::~ACMStream()
 {
         // release memory - encoding is finished
-	lame_close( gfp );
+	if (gfp) lame_close( gfp );
 
 	if (my_debug != NULL)
 	{
@@ -289,18 +289,19 @@ bool bResult = false;
 
 DWORD ACMStream::GetOutputSizeForInput(const DWORD the_SrcLength) const
 {
-	double OutputInputRatio;
+/*	double OutputInputRatio;
 
 	if (my_VBRMode == vbr_off)
 		OutputInputRatio = double(my_AvgBytesPerSec) / double(my_OutBytesPerSec * 2);
 	else // reserve the space for 320 kbps
 		OutputInputRatio = 40000.0 / double(my_OutBytesPerSec * 2);
 
-	OutputInputRatio *= 1.15; // allow 15% more
+	OutputInputRatio *= 1.15; // allow 15% more*/
 
-DWORD Result;
+    DWORD Result;
 
-	Result = DWORD(double(the_SrcLength) * OutputInputRatio);
+//	Result = DWORD(double(the_SrcLength) * OutputInputRatio);
+    Result = DWORD(1.25*the_SrcLength + 7200);
 
 my_debug->OutPut(DEBUG_LEVEL_FUNC_CODE, "Result = %d",Result);
 
