@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: lametime.c,v 1.13 2002/05/14 16:19:34 markt Exp $ */
+/* $Id: lametime.c,v 1.15 2003/05/09 14:06:41 aleidinger Exp $ */
 
 /*
  * name:        GetCPUTime ( void )
@@ -48,6 +48,11 @@
 
 #include "lametime.h"
 
+#if !defined(CLOCKS_PER_SEC)
+# warning Your system does not define CLOCKS_PER_SEC, guessing one...
+# define CLOCKS_PER_SEC 1000000
+#endif
+
 
 double GetCPUTime ( void )
 {
@@ -58,12 +63,6 @@ double GetCPUTime ( void )
 #else
     t = clock ();
 #endif    
-
-    /*
-fprintf(stderr,"clock_t t = %lli \n\n",(int64_t) t);
-fprintf(stderr,"output: %f \n\n",t/(double)CLOCKS_PER_SEC);
-    */
-
     return t / (double) CLOCKS_PER_SEC;
 }
 
@@ -160,3 +159,4 @@ off_t  lame_get_file_size ( const char* const filename )
 }
 
 /* End of lametime.c */
+
