@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: bitstream.c,v 1.60 2003/10/22 07:33:49 takehiro Exp $
+ * $Id: bitstream.c,v 1.66 2004/03/23 01:57:17 olcios Exp $
  */
 
 
@@ -35,9 +35,8 @@
 #include "quantize_pvt.h"
 #include "version.h"
 #include "VbrTag.h"
-#ifdef DECODE_ON_THE_FLY
+#include "machine.h"
 #include "gain_analysis.h"
-#endif
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
@@ -643,16 +642,12 @@ writeMainData ( lame_global_flags * const gfp)
 		for (sfb = 0; sfb < gi->sfbdivide; sfb++) {
 		    if (gi->scalefac[sfb] == -1)
 			continue; /* scfsi is used */
-		    if (gi->scalefac[sfb] == -2)
-			gi->scalefac[sfb] = 0;
 		    putbits2(gfc, gi->scalefac[sfb], slen1);
 		    data_bits += slen1;
 		}
 		for (; sfb < gi->sfbmax; sfb++) {
 		    if (gi->scalefac[sfb] == -1)
 			continue; /* scfsi is used */
-		    if (gi->scalefac[sfb] == -2)
-			gi->scalefac[sfb] = 0;
 		    putbits2(gfc, gi->scalefac[sfb], slen2);
 		    data_bits += slen2;
 		}
@@ -825,6 +820,31 @@ flush_bitstream(lame_global_flags *gfp)
   gfc->ResvSize=0;
   l3_side->main_data_begin = 0;
 
+
+  /* save the ReplayGain value */
+  if (gfp->findReplayGain) {
+    FLOAT RadioGain = (FLOAT) GetTitleGain(gfc->rgdata);
+    assert(RadioGain != GAIN_NOT_ENOUGH_SAMPLES); 
+    gfp->internal_flags->RadioGain = (int) floor( RadioGain * 10.0 + 0.5 ); /* round to nearest */
+  }
+
+  /* find the gain and scale change required for no clipping */
+  if (gfc->findPeakSample) {
+    gfc->noclipGainChange = (int) ceil(log10(gfc->PeakSample / 32767.0) *20.0*10.0);  /* round up */
+    
+    if (gfc->noclipGainChange > 0.0) { /* clipping occurs */
+      if (gfp->scale == 1.0 || gfp->scale == 0.0) 
+	gfc->noclipScale = floor( (32767.0 / gfc->PeakSample) * 100.0 ) / 100.0;  /* round down */
+      else
+        /* the user specified his own scaling factor. We could suggest 
+         * the scaling factor of (32767.0/gfp->PeakSample)*(gfp->scale)
+         * but it's usually very inaccurate. So we'd rather not advice him
+         * on the scaling factor. */
+        gfc->noclipScale = -1;
+    }
+    else  /* no clipping */
+      gfc->noclipScale = -1;
+  }
 }
 
 
@@ -949,54 +969,60 @@ int copy_buffer(lame_internal_flags *gfc,unsigned char *buffer,int size,int mp3d
         UpdateMusicCRC(&gfc->nMusicCRC,buffer,minimum);
 
 #ifdef DECODE_ON_THE_FLY 
-        if (gfc->decode_on_the_fly) {  /* decode the frame */
-          sample_t pcm_out[2][1152];
-          int mp3out;
+        if (gfc->gfp->decode_on_the_fly) {  /* decode the frame */
+          sample_t pcm_buf[2][1152];
+	  int mp3_in = minimum;
+          int samples_out = -1;
           int i;
 
-          /* re-synthesis to pcm.  Repeat until we get a mp3out=0 */
-          mp3out=lame_decode1_unclipped(buffer,minimum,pcm_out[0],pcm_out[1]); 
-          /* mp3out = 0:  need more data to decode */
-          /* mp3out = -1:  error.  Lets assume 0 pcm output */
-          /* mp3out = number of samples output */
+          /* re-synthesis to pcm.  Repeat until we get a samples_out=0 */
+          while(samples_out != 0) {
+
+            samples_out=lame_decode1_unclipped(buffer,mp3_in,pcm_buf[0],pcm_buf[1]); 
+            /* samples_out = 0:  need more data to decode 
+             * samples_out = -1:  error.  Lets assume 0 pcm output 
+             * samples_out = number of samples output */
+	    
+	    /* set the lenght of the mp3 input buffer to zero, so that in the 
+	     * next iteration of the loop we will be querying mpglib about 
+	     * buffered data */
+	    mp3_in = 0;
               
-          if (mp3out==-1) {
-              /* error decoding. Not fatal, but might screw up */
-              /* the ReplayGain tag. what should we do? ignore for now */
-              mp3out=0;
-          }
-          if (mp3out>0) {  
-            /* process the PCM data */
-
-   /*     if (mp3out>1152) {
-    *         this should not be possible, and indicates we have 
-    *         overflown the pcm_out buffer.  Fatal error. 
-    *       return -6;
-    *     } 
-    */
-
-            if (gfc->gfp->findPeakSample) {
-              for (i=0; i<mp3out; i++) {   
-                 if (pcm_out[0][i] > gfc->PeakSample)
-                   gfc->PeakSample = pcm_out[0][i];
-                 else if (-pcm_out[0][i] > gfc->PeakSample)
-                   gfc->PeakSample = -pcm_out[0][i]; 
-              }
-              if (gfc->channels_out > 1)
-                for (i=0; i<mp3out; i++) {
-                  if (pcm_out[1][i] > gfc->PeakSample)
-                    gfc->PeakSample = pcm_out[1][i];
-                  else if (-pcm_out[1][i] > gfc->PeakSample)
-                    gfc->PeakSample = -pcm_out[1][i];
-                }
+            if (samples_out==-1) {
+                /* error decoding. Not fatal, but might screw up 
+                 * the ReplayGain tag. What should we do? Ignore for now */
+                samples_out=0;
             }
+            if (samples_out>0) {  
+              /* process the PCM data */
 
-            if (gfc->gfp->ReplayGain_decode)
-              if (AnalyzeSamples(pcm_out[0], pcm_out[1], mp3out, gfc->channels_out) == GAIN_ANALYSIS_ERROR)
-                 return -6;
+              /* this should not be possible, and indicates we have 
+               * overflown the pcm_buf buffer */
+              assert(samples_out <= 1152);
 
-          } /* if (mp3out>0) */
-        } /* if (gfc->decode_on_the_fly) */ 
+              if (gfc->findPeakSample) {
+                for (i=0; i<samples_out; i++) {   
+                   if (pcm_buf[0][i] > gfc->PeakSample)
+                     gfc->PeakSample = pcm_buf[0][i];
+                   else if (-pcm_buf[0][i] > gfc->PeakSample)
+                     gfc->PeakSample = -pcm_buf[0][i]; 
+                }
+                if (gfc->channels_out > 1)
+                  for (i=0; i<samples_out; i++) {
+                    if (pcm_buf[1][i] > gfc->PeakSample)
+                      gfc->PeakSample = pcm_buf[1][i];
+                    else if (-pcm_buf[1][i] > gfc->PeakSample)
+                      gfc->PeakSample = -pcm_buf[1][i];
+                  }
+              }
+
+              if (gfc->gfp->findReplayGain)
+                if (AnalyzeSamples(gfc->rgdata, pcm_buf[0], pcm_buf[1], samples_out, gfc->channels_out) == GAIN_ANALYSIS_ERROR)
+                   return -6;
+		   
+            } /* if (samples_out>0) */
+          } /* while (samples_out!=0) */
+        } /* if (gfp->decode_on_the_fly) */ 
 #endif
   
     } /* if (mp3data) */
