@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: lame.c,v 1.247 2004/01/11 15:01:43 bouvigne Exp $ */
+/* $Id: lame.c,v 1.261 2004/03/23 01:57:17 olcios Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -37,6 +37,7 @@
 #include "quantize_pvt.h"
 #include "psymodel.h"
 #include "VbrTag.h"
+#include "machine.h"
 #include "gain_analysis.h"
 #include "set_get.h"
 
@@ -396,7 +397,7 @@ lame_init_qval(lame_global_flags * gfp)
         gfc->use_best_huffman = 2;
         break;
     }
-
+    
 }
 
 
@@ -506,6 +507,11 @@ lame_init_params(lame_global_flags * const gfp)
     if (NULL == gfc->PSY)
         return -2;
         
+    if (NULL == gfc->rgdata)
+        gfc->rgdata = calloc(1, sizeof(replaygain_t));
+    if (NULL == gfc->rgdata)
+        return -2;
+        
     gfc->channels_in = gfp->num_channels;
     if (gfc->channels_in == 1)
         gfp->mode = MONO;
@@ -643,48 +649,27 @@ lame_init_params(lame_global_flags * const gfp)
                                                             VBR_mean_bitrate_kbps);
     }
 
-
-    /*do not compute ReplayGain values if we can't store them*/
+    /* do not compute ReplayGain values and do not find the peak sample
+       if we can't store them */
     if (!gfp->bWriteVbrTag){
-	gfp->ReplayGain_input = 0;
-#ifdef DECODE_ON_THE_FLY
-        gfp->ReplayGain_decode = 0;
-#endif
+        gfp->findReplayGain = 0;
+	gfp->decode_on_the_fly = 0;
+        gfc->findPeakSample = 0;
     }
 
+    if (gfp->decode_on_the_fly) 
+      gfc->findPeakSample = 1;
 
-    if (gfp->ReplayGain_input)
-        gfc->findReplayGain = 1;
-#ifdef DECODE_ON_THE_FLY
-    if (gfp->ReplayGain_decode) {
-        gfp->ReplayGain_input = 0;
-        gfc->findReplayGain = 1;
-	gfc->decode_on_the_fly = 1;
-    }
-#endif
-
-    if (gfp->ReplayGain_input) {
-      if (InitGainAnalysis(gfp->out_samplerate) == INIT_GAIN_ANALYSIS_ERROR)
+    if (gfp->findReplayGain) {
+      if (InitGainAnalysis(gfc->rgdata, gfp->out_samplerate) == INIT_GAIN_ANALYSIS_ERROR)
         return -6;
     }
-#ifdef DECODE_ON_THE_FLY
-    else {
-      if (gfp->ReplayGain_decode) {
-        if (InitGainAnalysis(gfp->out_samplerate) == INIT_GAIN_ANALYSIS_ERROR)
-          return -6;
-      }
-    }
-#endif
 
 #ifdef DECODE_ON_THE_FLY
-    if (gfp->findPeakSample)
-      gfc->decode_on_the_fly = 1;
-
-    if (gfc->decode_on_the_fly && !gfp->decode_only)
+    if (gfp->decode_on_the_fly && !gfp->decode_only)
       lame_decode_init();  /* initialize the decoder  */
 #endif
 
-
     gfc->mode_gr = gfp->out_samplerate <= 24000 ? 1 : 2; /* Number of granules per frame */
     gfp->framesize = 576 * gfc->mode_gr;
     gfp->encoder_delay = ENCDELAY;
@@ -1060,14 +1045,12 @@ lame_init_params(lame_global_flags * const gfp)
       it is a rare event (1 frame per min. or so) that LAME would use
       uncoupled short blocks, so lets turn them off until we decide
       how to handle this.  No other encoders allow uncoupled short blocks,
-      even though it is in the standard.  
-    if (gfp->short_blocks == short_block_allowed && gfp->mode == JOINT_STEREO) {
+      even though it is in the standard.  */
+    /* rh 20040217: coupling makes no sense for mono and dual-mono streams
+     */
+    if (gfp->short_blocks == short_block_allowed && (gfp->mode == JOINT_STEREO || gfp->mode == STEREO) ) {
         gfp->short_blocks =  short_block_coupled;
     } 
-    */
-    if (gfp->short_blocks == short_block_allowed) {
-        gfp->short_blocks =  short_block_coupled;
-    }    
 
     
     if (lame_get_quant_comp(gfp) < 0 ) lame_set_quant_comp(gfp, 1);
@@ -1195,7 +1178,7 @@ lame_print_config(const lame_global_flags * gfp)
         if (0. < gfc->lowpass1 && gfc->lowpass1 < 1.) {
             MSGF
                 (gfc,
-                 "Using polyphase lowpass  filter, transition band: %5.0f Hz - %5.0f Hz\n",
+                 "Using polyphase lowpass filter, transition band: %5.0f Hz - %5.0f Hz\n",
                  0.5 * gfc->lowpass1 * out_samplerate,
                  0.5 * gfc->lowpass2 * out_samplerate);
         }
@@ -1456,7 +1439,7 @@ lame_encode_buffer_sample_t(lame_global_flags * gfp,
 #endif
 
     mf_needed = BLKSIZE + gfp->framesize - FFTOFFSET; /* amount needed for FFT */
-    //mf_needed = Max(mf_needed, 286 + 576 * (1 + gfc->mode_gr)); 
+    /*mf_needed = Max(mf_needed, 286 + 576 * (1 + gfc->mode_gr)); */
     mf_needed = Max(mf_needed, 512+gfp->framesize-32 );
 
     assert(MFSIZE >= mf_needed);
@@ -1473,8 +1456,8 @@ lame_encode_buffer_sample_t(lame_global_flags * gfp,
         fill_buffer(gfp, mfbuf, in_buffer, nsamples, &n_in, &n_out);
 
         /* compute ReplayGain of resampled input if requested */
-        if (gfp->ReplayGain_input) 
-            if (AnalyzeSamples(mfbuf[0], mfbuf[1], n_out, gfc->channels_out) == GAIN_ANALYSIS_ERROR) 
+        if (gfp->findReplayGain && !gfp->decode_on_the_fly) 
+            if (AnalyzeSamples(gfc->rgdata, &mfbuf[0][gfc->mf_size], &mfbuf[1][gfc->mf_size], n_out, gfc->channels_out) == GAIN_ANALYSIS_ERROR) 
                 return -6;
 
 
@@ -1507,7 +1490,7 @@ lame_encode_buffer_sample_t(lame_global_flags * gfp,
             ret =
                 lame_encode_frame(gfp, mfbuf[0], mfbuf[1], mp3buf,buf_size);
 
-            if (ret < 0) goto retr;
+            if (ret < 0) return ret;
             mp3buf += ret;
             mp3size += ret;
 
@@ -1520,10 +1503,8 @@ lame_encode_buffer_sample_t(lame_global_flags * gfp,
         }
     }
     assert(nsamples == 0);
-    ret = mp3size;
 
-  retr:
-    return ret;
+    return mp3size;
 }
 
 
@@ -1821,9 +1802,7 @@ lame_init_bitstream(lame_global_flags * gfp)
 	   sizeof(gfc->bitrate_blockType_Hist));
 #endif
 
-#ifdef DECODE_ON_THE_FLY
     gfc->PeakSample = 0.0;
-#endif
 
     /* Write initial VBR Header to bitstream and init VBR data */
     if (gfp->bWriteVbrTag) 
@@ -1973,13 +1952,6 @@ lame_encode_finish(lame_global_flags * gfp,
 void
 lame_mp3_tags_fid(lame_global_flags * gfp, FILE * fpStream)
 {
-
-    if (gfp->internal_flags->findReplayGain) {
-        FLOAT RadioGain = (FLOAT) GetTitleGain();
-        assert(RadioGain != GAIN_NOT_ENOUGH_SAMPLES); 
-        gfp->internal_flags->RadioGain = (int) floor( RadioGain * 10.0 + 0.5 ); /* round to nearest */
-    }
-
     if (gfp->bWriteVbrTag) {
         /* Map VBR_q to Xing quality value: 0=worst, 100=best */
         int     nQuality = ((9-gfp->VBR_q) * 100) / 9;
@@ -1988,7 +1960,6 @@ lame_mp3_tags_fid(lame_global_flags * gfp, FILE * fpStream)
         if (fpStream && !fseek(fpStream, 0, SEEK_SET))
             PutVbrTag(gfp, fpStream, nQuality);
     }
-
 }
 
 lame_global_flags *
@@ -2102,18 +2073,15 @@ lame_init_old(lame_global_flags * gfp)
     gfp->encoder_padding = 0;
     gfc->mf_size = ENCDELAY - MDCTDELAY; /* we pad input with this many 0's */
 
-#ifdef DECODE_ON_THE_FLY
-    gfc->decode_on_the_fly = 0;
-    gfp->findPeakSample = 0;
-#endif
+    gfp->findReplayGain = 0;
+    gfp->decode_on_the_fly = 0;
+
+    gfc->findPeakSample = 0;
 
-    gfc->findReplayGain = 0;
-    gfp->ReplayGain_input = 1;
-#ifdef DECODE_ON_THE_FLY
-    gfp->ReplayGain_decode = 0;
-#endif
     gfc->RadioGain = 0.0;
     gfc->AudiophileGain = 0.0;
+    gfc->noclipGainChange = 0.0;
+    gfc->noclipScale = -1;
 
     gfp->asm_optimizations.mmx = 1;
     gfp->asm_optimizations.amd3dnow = 1;
