@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: psymodel.c,v 1.131 2003/12/21 18:14:13 bouvigne Exp $ */
+/* $Id: psymodel.c,v 1.135 2004/03/01 22:20:37 robert Exp $ */
 
 
 /*
@@ -482,11 +482,15 @@ static void convert_partition2scalefac_l(
 	if (sb == SBMAX_l - 1)
 	    break;
 
+    assert( enn >= 0 );
+    assert( thmm >= 0 );
 	gfc->en [chn].l[sb] = enn  + 0.5 * eb [b];
 	gfc->thm[chn].l[sb] = thmm + 0.5 * thr[b];
 
 	enn  = 0.5 *  eb[b];
 	thmm = 0.5 * thr[b];
+    assert( enn >= 0 );
+    assert( thmm >= 0 );
 	b++;
 	sb++;
     }
@@ -528,6 +532,7 @@ compute_masking_s(
 	thr[b] = Max( thr[b], gfc->ATH->cb[gfc->bm_s[b]] * athlower );
 	gfc->nb_s2[chn][b] = gfc->nb_s1[chn][b];
 	gfc->nb_s1[chn][b] = ecb;
+    assert( thr[b] >= 0 );
     }
 }
 
@@ -598,9 +603,9 @@ int L3psycho_anal( lame_global_flags * gfp,
     FLOAT fftenergy_s[3][HBLKSIZE_s];
 
     /* convolution   */
-    FLOAT8 eb[CBANDS];
+    FLOAT8 eb[CBANDS+1];
     FLOAT8 cb[CBANDS];
-    FLOAT8 thr[CBANDS];
+    FLOAT8 thr[CBANDS+1];
 
     /* ratios    */
     FLOAT8 ms_ratio_l=0, ms_ratio_s=0;
@@ -613,6 +618,17 @@ int L3psycho_anal( lame_global_flags * gfp,
     int b, i, j, k;
     int sb,sblock;
 
+    /*  rh 20040301: the following loops do access one off the limits
+     *  so I increase  the array dimensions by one and initialize the
+     *  accessed values to zero
+     */
+    assert( gfc->npart_s <= CBANDS );
+    assert( gfc->npart_l <= CBANDS );
+    eb [gfc->npart_s] = 0;
+    thr[gfc->npart_s] = 0;
+    eb [gfc->npart_l] = 0;
+    thr[gfc->npart_l] = 0;
+
     numchn = gfc->channels_out;
     /* chn=2 and 3 = Mid and Side channels */
     if (gfp->mode == JOINT_STEREO) numchn=4;
@@ -921,12 +937,16 @@ int L3psycho_anal( lame_global_flags * gfp,
 		    enn  += eb[b];
 		    thmm += thr[b];
 		}
-		enn  += 0.5 * eb[b];
-		thmm += 0.5 * thr[b];
+		enn  += 0.5 * eb[b];    /* for the last sfb b is larger than npart_s!! */
+		thmm += 0.5 * thr[b];   /* rh 20040301 */
+        assert( enn >= 0 );
+        assert( thmm >= 0 );
 		gfc->en [chn].s[sb][sblock] = enn;
 		gfc->thm[chn].s[sb][sblock] = thmm;
 		enn  = 0.5 * eb[b];
 		thmm = 0.5 * thr[b];
+        assert( enn >= 0 );
+        assert( thmm >= 0 );
 	    }
 	    gfc->en [chn].s[sb-1][sblock] += enn;
 	    gfc->thm[chn].s[sb-1][sblock] += thmm;
@@ -1058,10 +1078,12 @@ inline static FLOAT8 mask_add(FLOAT8 m1,FLOAT8 m2,int k,int b, lame_internal_fla
 
 
   if (m2 > m1) {
-      if (m1 == 0) return m2;
+      if (m2 >= (m1*ma_max_i2))
+          return (m1+m2);
       ratio = m2/m1;
   } else {
-      if (m2 == 0) return m1;
+      if (m1 >= (m2*ma_max_i2))
+          return (m1+m2);
       ratio = m1/m2;
   }
   /*i = abs(10*log10(m2 / m1)/10*16);
@@ -1075,6 +1097,7 @@ inline static FLOAT8 mask_add(FLOAT8 m1,FLOAT8 m2,int k,int b, lame_internal_fla
 
 
   m1 += m2;
+
   if ((unsigned int)(b+3) <= 3+3) {  /* approximately, 1 bark = 3 partitions */
       /* 65% of the cases */
       /* originally 'if(i > 8)' */
@@ -1093,9 +1116,6 @@ inline static FLOAT8 mask_add(FLOAT8 m1,FLOAT8 m2,int k,int b, lame_internal_fla
    * equ (m1+m2)<10^1.5 * gfc->ATH->cb[k]
    */
 
-  if (ratio >= ma_max_i2)
-      return m1;
-
   i = FAST_LOG10_X(ratio, 16.0);
   m2 = gfc->ATH->cb[k]*gfc->ATH->adjust;
   if (m1 < ma_max_m*m2)  {
@@ -1128,7 +1148,7 @@ static inline FLOAT8 NS_INTERP(FLOAT8 x, FLOAT8 y, FLOAT8 r)
     /* was pow((x),(r))*pow((y),1-(r))*/
     if(r==1.0)
         return x;              /* 99.7% of the time */ 
-    if(y!=0.0)
+    if(y>0.0)
         return pow(x/y,r)*y;   /* rest of the time */ 
     return 0.0;                /* never happens */ 
 }
@@ -1185,7 +1205,7 @@ pecalc_s(
 	84.6,
 	97.6,
 	130,
-//	255.8
+/*	255.8 */
     };
     int sb, sblock;
 
@@ -1238,7 +1258,7 @@ pecalc_l(
 	85.7,
 	93.4,
 	126.1,
-//	241.3
+/*	241.3 */
     };
     int sb;
 
@@ -1353,8 +1373,8 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	FLOAT fftenergy[HBLKSIZE];
 	FLOAT fftenergy_s[3][HBLKSIZE_s];
 	/* convolution   */
-	FLOAT8 eb[CBANDS],eb2[CBANDS];
-	FLOAT8 thr[CBANDS];
+	FLOAT8 eb[CBANDS+1],eb2[CBANDS];
+	FLOAT8 thr[CBANDS+1];
 
 
     /*This is the masking table:
@@ -1375,6 +1395,17 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	    0.11749/*pow(10, -0.93)*/
 	};
 
+    /*  rh 20040301: the following loops do access one off the limits
+     *  so I increase  the array dimensions by one and initialize the
+     *  accessed values to zero
+     */
+    assert( gfc->npart_s <= CBANDS );
+    assert( gfc->npart_l <= CBANDS );
+    eb [gfc->npart_s] = 0;
+    thr[gfc->npart_s] = 0;
+    eb [gfc->npart_l] = 0;
+    thr[gfc->npart_l] = 0;
+    
 	/*************************************************************** 
 	 * determine the block type (window type)
 	 ***************************************************************/
@@ -1460,17 +1491,20 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	    FLOAT8 enn, thmm;
 	    compute_masking_s(gfc, fftenergy_s, eb, thr, chn, sblock,
 			      gfp->ATHlower*gfc->ATH->adjust);
-	    b = -1;
+        b = -1;
 	    for (sb = 0; sb < SBMAX_s; sb++) {
 		enn = thmm = 0.0;
 		while (++b < gfc->bo_s[sb]) {
 		    enn  += eb[b];
 		    thmm += thr[b];
 		}
-		enn  += 0.5 * eb[b];
-		thmm += 0.5 * thr[b];
+		enn  += 0.5 * eb[b];    /* for the last sfb b is larger than npart_s!! */
+		thmm += 0.5 * thr[b];   /* rh 20040301 */
 		gfc->en [chn].s[sb][sblock] = enn;
-
+        
+        assert( enn >= 0 );
+        assert( thmm >= 0 );
+        
 		/****   short block pre-echo control   ****/
 		thmm *= NS_PREECHO_ATT0;
 		if (ns_attacks[sblock] >= 2 || ns_attacks[sblock+1] == 1) {
@@ -1909,7 +1943,7 @@ int psymodel_init(lame_global_flags *gfp)
 	for (k=0; k < gfc->numlines_l[i]; k++, j++) {
 	    FLOAT8  freq = sfreq*j/(1000.0*BLKSIZE);
 	    FLOAT8  level;
-	    /*	freq = Min(.1,freq);       // ATH below 100 Hz constant, not further climbing */
+	    /*	freq = Min(.1,freq);*/      /* ATH below 100 Hz constant, not further climbing */
 	    level  = ATHformula (freq*1000, gfp) - 20;   /* scale to FFT units; returned value is in dB */
 	    level  = pow ( 10., 0.1*level );   /* convert from dB -> energy */
 	    level *= gfc->numlines_l [i];
