@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: util.c,v 1.111 2003/12/21 10:16:36 bouvigne Exp $ */
+/* $Id: util.c,v 1.115 2004/03/23 22:11:11 robert Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -85,6 +85,9 @@ void  freegfc ( lame_internal_flags* const gfc )   /* bit stream structure */
     if ( gfc->PSY ) {
         free ( gfc->PSY );
     }
+    if ( gfc->rgdata ) {
+        free ( gfc->rgdata );
+    }
     if ( gfc->s3_ll ) {
         /* XXX allocated in psymodel_init() */
         free ( gfc->s3_ll );
@@ -134,8 +137,8 @@ bitrate is more balanced according to the -V value.*/
 
   f /= 1000;  /* convert to khz */
   f  = Max(0.01, f);
-//  f  = Min(21.0, f);
-
+/*  f  = Min(21.0, f);
+*/
   ath =    3.640 * pow(f,-0.8)
          - 6.800 * exp(-0.6*pow(f-3.4,2.0))
          + 6.000 * exp(-0.15*pow(f-8.7,2.0))
@@ -226,7 +229,7 @@ int nearestBitrateFullIndex(const int bitrate)
 {
     /* borrowed from DM abr presets*/    
 
-    int index; // resolved range
+    int index; /* resolved range */
 
     const int bitrate_table[] = {8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320};
 
@@ -238,25 +241,26 @@ int nearestBitrateFullIndex(const int bitrate)
     int b;
 
 
-    // We assume specified bitrate will be 320kbps
+    /* We assume specified bitrate will be 320kbps */
     upper_range_kbps = bitrate_table[16];
     upper_range = 16;
     lower_range_kbps = bitrate_table[16];
     lower_range = 16;
  
-    // Determine which significant bitrates the value specified falls between,
-    // if loop ends without breaking then we were correct above that the value was 320
+    /* Determine which significant bitrates the value specified falls between,
+     * if loop ends without breaking then we were correct above that the value was 320
+     */
     for (b = 0; b < 16; b++) {
         if ((Max(bitrate, bitrate_table[b+1])) != bitrate) {
               upper_range_kbps = bitrate_table[b+1];
               upper_range = b+1;
               lower_range_kbps = bitrate_table[b];
               lower_range = (b);
-              break; // We found upper range 
+              break; /* We found upper range */
         }
     }
 
-    // Determine which range the value specified is closer to
+    /* Determine which range the value specified is closer to */
     if ((upper_range_kbps - bitrate) > (bitrate - lower_range_kbps))
         index = lower_range;
     else
@@ -484,7 +488,7 @@ int fill_buffer_resample(
     xvalue = 0.;
     for (i=0 ; i<=filter_l ; ++i) {
       int j2 = i+j-filter_l/2;
-      int y;
+      sample_t y;
       assert(j2<len);
       assert(j2+BLACKSIZE >= 0);
       y = (j2<0) ? inbuf_old[BLACKSIZE+j2] : inbuf[j2];
@@ -786,12 +790,15 @@ void init_log_table(void)
 
 ieee754_float32_t fast_log2(ieee754_float32_t x)
 {
-  int i = *(int*)&x;
-  ieee754_float32_t log2val;
-  int mantisse = i & 0x7FFFFF;
-  ieee754_float32_t partial;
-
-  log2val = ((i>>23) & 0xFF)-0x7f;
+  ieee754_float32_t log2val, partial;
+  union {
+    ieee754_float32_t f;
+    int     i;
+  } fi;
+  int mantisse;
+  fi.f = x;
+  mantisse = fi.i & 0x7fffff;
+  log2val = ((fi.i>>23) & 0xFF)-0x7f;
   partial = (mantisse & ((1<<(23-LOG2_SIZE_L2))-1));
   partial *= 1.0f/((1<<(23-LOG2_SIZE_L2)));
 
