@@ -5,7 +5,7 @@
  * All rights reserved. See also 'README'
  */ 
 
-/* $Id: layer3.c,v 1.43 2001/06/04 10:01:35 afaber Exp $ */
+/* $Id: layer3.c,v 1.47 2004/01/10 10:27:28 takehiro Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -482,8 +482,12 @@ static void III_get_side_info_2(struct III_sideinfo *si,int stereo,
          }
          /* region_count/start parameters are implicit in this case. */       
 /* check this again! */
-         if(gr_infos->block_type == 2)
-           gr_infos->region1start = 36>>1;
+         if(gr_infos->block_type == 2) {
+            if (sfreq == 8)
+                gr_infos->region1start = 36;
+            else
+                gr_infos->region1start = 36>>1;
+         }
          else if(sfreq == 8)
 /* check this for 2.5 and sfreq=8 */
            gr_infos->region1start = 108>>1;
@@ -699,7 +703,18 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
       }
     }
   }
- 
+  /* MDH crash fix */
+  {
+      int i;
+      for (i = 0; i < 3; i++) {
+	  if (l[i] < 0) {
+	      fprintf(stderr, "mpg123: Bogus region length (%d)\n", l[i]);
+	      l[i] = 0;
+	  }
+      }
+  }
+  /* end MDH crash fix */
+
   if(gr_infos->block_type == 2) {
     /*
      * decoding with short or mixed mode BandIndex table 
@@ -1568,11 +1583,13 @@ int do_layer3_sideinfo(struct frame *fr)
 
 
 
-int do_layer3( PMPSTR mp,unsigned char *pcm_sample,int *pcm_point)
+int  do_layer3( PMPSTR mp,unsigned char *pcm_sample,int *pcm_point,
+                int (*synth_1to1_mono_ptr)(PMPSTR,real *,unsigned char *,int *),
+                int (*synth_1to1_ptr)(PMPSTR,real *,int,unsigned char *, int *) )
 {
   int gr, ch, ss,clip=0;
   int scalefacs[2][39]; /* max 39 for short[13][3] mode, mixed: 38, long: 22 */
-  //  struct III_sideinfo sideinfo;
+  /*  struct III_sideinfo sideinfo; */
   struct frame *fr=&(mp->fr);
   int stereo = fr->stereo;
   int single = fr->single;
@@ -1780,12 +1797,12 @@ int do_layer3( PMPSTR mp,unsigned char *pcm_sample,int *pcm_point)
 
     for(ss=0;ss<SSLIMIT;ss++) {
       if(single >= 0) {
-        clip += synth_1to1_mono(mp, hybridOut[0][ss],pcm_sample,pcm_point);
+        clip += (*synth_1to1_mono_ptr)(mp, hybridOut[0][ss],pcm_sample,pcm_point);
       }
       else {
         int p1 = *pcm_point;
-        clip += synth_1to1(mp, hybridOut[0][ss],0,pcm_sample,&p1);
-        clip += synth_1to1(mp, hybridOut[1][ss],1,pcm_sample,pcm_point);
+        clip += (*synth_1to1_ptr)(mp, hybridOut[0][ss],0,pcm_sample,&p1);
+        clip += (*synth_1to1_ptr)(mp, hybridOut[1][ss],1,pcm_sample,pcm_point);
       }
     }
   }
@@ -1793,3 +1810,4 @@ int do_layer3( PMPSTR mp,unsigned char *pcm_sample,int *pcm_point)
   return clip;
 }
 
+
